<?php
/**
 * Services: Cached Entity Uri Service
 *
 * @since      3.16.3
 * @package    Wordlift
 * @subpackage Wordlift/includes/cache
 */

/**
 * Define the {@link Wordlift_Cached_Entity_Uri_Service} instance.
 *
 * @since 3.16.3
 */
class Wordlift_Cached_Entity_Uri_Service extends Wordlift_Entity_Uri_Service {

	/**
	 * The {@link Wordlift_Cache_Service} instance.
	 *
	 * @since  3.16.3
	 * @access private
	 * @var \Wordlift_Cache_Service $cache_service The {@link Wordlift_Cache_Service} instance.
	 */
	private $cache_service;

	/**
	 * A {@link Wordlift_Log_Service} instance.
	 *
	 * @since  3.16.3
	 * @access private
	 * @var \Wordlift_Log_Service $log A {@link Wordlift_Log_Service} instance.
	 */
	private $log;

	/**
	 * Create a {@link Wordlift_Cached_Entity_Uri_Service} instance.
	 *
	 * @since 3.16.3
	 *
	 * @param \Wordlift_Configuration_Service $configuration_service
	 * @param \Wordlift_Cache_Service         $cache_service
	 */
	public function __construct( $configuration_service, $cache_service ) {
		parent::__construct( $configuration_service );

		$this->log = Wordlift_Log_Service::get_logger( get_class() );

		// Add hooks for meta being added/modified/deleted.
		$this->cache_service = $cache_service;

		add_action( 'add_post_meta', array(
			$this,
			'on_before_post_meta_add',
		), 10, 3 );
		add_action( 'update_post_meta', array(
			$this,
			'on_before_post_meta_change',
		), 10, 4 );
		add_action( 'delete_post_meta', array(
			$this,
			'on_before_post_meta_change',
		), 10, 4 );

	}

	/**
	 * Preload the URIs.
	 *
	 * @since 3.16.3
	 *
	 * @param array $uris Preload an array of URIs.
	 */
	public function preload_uris( $uris ) {

		// Filter the URIs which aren't yet cached.
		$cache_service = $this->cache_service;
		$uris_to_cache = array_filter( (array) $uris, function ( $item ) use ( $cache_service ) {
			return ! $cache_service->has_cache( $item );
		} );

		// Preload the URIs.
		parent::preload_uris( $uris_to_cache );

		// Store them in cache.
		if ( is_array( $this->uri_to_post ) && ! empty( $this->uri_to_post ) ) {
			foreach ( $this->uri_to_post as $uri => $post ) {
				$this->set_cache( $uri, $post );
			}
		}

	}

	/**
	 * Get the entity post for the specified URI.
	 *
	 * @since 3.16.3
	 *
	 * @param string $uri The URI.
	 *
	 * @return null|WP_Post The {@link WP_Post} or null if not found.
	 */
	public function get_entity( $uri ) {

		$this->log->trace( "Getting entity for uri $uri..." );

		// Get the cached post for the specified URI.
		$cache = $this->cache_service->get_cache( $uri );

		// Return the cached data if valid.
		if ( false !== $cache && is_numeric( $cache ) ) {
			$this->log->debug( "Cached entity $cache for uri $uri found." );

			return get_post( $cache );
		}

		// Get the actual result.
		$post = parent::get_entity( $uri );

		// Cache the result, if a post is not found we cache -1.
		$this->set_cache( $uri, $post );

		// Return the result.
		return $post;
	}

	/**
	 * Set the cached URI for the specified {@link WP_Post}.
	 *
	 * @since 3.16.3
	 *
	 * @param string  $uri  The URI.
	 * @param WP_Post $post The post.
	 */
	private function set_cache( $uri, $post ) {

		// Cache the result, if a post is not found we cache -1.
		$this->cache_service->set_cache( $uri, ( null === $post ? - 1 : $post->ID ) );

	}

	/**
	 * Delete the cache for the specified URIs.
	 *
	 * @since 3.16.3
	 *
	 * @param array $uris An array of URIs.
	 */
	private function delete_cache( $uris ) {

		// Delete the cache for each URI.
		foreach ( $uris as $uri ) {
			// Delete the single cache file.
			$this->cache_service->delete_cache( $uri );
		}

	}

	/**
	 * Before post meta change.
	 *
	 * When a post meta is going to be changed, we check if the `meta_key` is
	 * either the `entity_url` or the `same_as` in which case we delete the cache
	 * for all the associated URIs.
	 *
	 * @since 3.16.3
	 *
	 * @param int|array $meta_ids   The {@link WP_Post} meta id(s).
	 * @param int       $post_id    The {@link WP_Post} id.
	 * @param string    $meta_key   The meta key.
	 * @param mixed     $meta_value The meta value(s).
	 */
	public function on_before_post_meta_change( $meta_ids, $post_id, $meta_key, $meta_value ) {

		// Bail out if we're not interested in the meta key.
		if ( WL_ENTITY_URL_META_NAME !== $meta_key && Wordlift_Schema_Service::FIELD_SAME_AS !== $meta_key ) {
			return;
		}

		$this->log->trace( "Updating/deleting $meta_key for post $post_id ($meta_key), invalidating cache..." );

		// The list of existing URIs, plus the list of URIs being deleted/updated.
		$uris = get_post_meta( $post_id, $meta_key ) + (array) $meta_value;

		// Delete the cache for those URIs.
		$this->delete_cache( $uris );

	}

	/**
	 * Hook to meta add for a {@link WP_Post}, will cause the cache to
	 * invalidate.
	 *
	 * @since 3.16.3
	 *
	 * @param int    $post_id    The {@link WP_Post} id.
	 * @param string $meta_key   The meta key.
	 * @param mixed  $meta_value The meta value(s).
	 */
	public function on_before_post_meta_add( $post_id, $meta_key, $meta_value ) {

		// Bail out if we're not interested in the meta key.
		if ( WL_ENTITY_URL_META_NAME !== $meta_key && Wordlift_Schema_Service::FIELD_SAME_AS !== $meta_key ) {
			return;
		}

		$this->log->trace( "Adding $meta_key for post $post_id ($meta_key), invalidating cache..." );

		// Delete the cache for the URIs being added.
		$this->delete_cache( (array) $meta_value );

	}

}